# -*- coding: utf-8 -*-
# Copyright (c) Polyconseil SAS. All rights reserved.

from __future__ import absolute_import
from __future__ import unicode_literals

import pkg_resources

__version__ = pkg_resources.get_distribution('grocker').version
__copyright__ = '2015, Polyconseil'
